/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch das
 * Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.eingang.xta;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.xta.zip.ZipFileExtractor;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
@RequiredArgsConstructor
class XtaIncomingFilesMapper {

	private final ZipFileExtractor zipFileExtractor;

	public List<IncomingFile> toIncomingFiles(Collection<XtaFile> messageFiles) {
		if (Objects.nonNull(messageFiles)) {
			return messageFiles.stream()
					.map(this::toIncomingFile)
					.flatMap(this::tryToExtractZip)
					.toList();
		}
		return List.of();
	}

	IncomingFile toIncomingFile(XtaFile messageFile) {
		return IncomingFile.builder()
				.name(messageFile.name())
				.contentType(messageFile.contentType())
				.file(messageFile.file())
				.size(messageFile.file().length())
				.build();
	}

	Stream<IncomingFile> tryToExtractZip(IncomingFile incomingFile) {
		try {
			List<IncomingFile> extractedZips = zipFileExtractor.extractIncomingFilesSafely(incomingFile);
			return extractedZips.stream();
		} catch (RuntimeException e) {
			return Stream.of(incomingFile);
		}
	}
}
